# LiveShot data format
There is one primary data types used in LiveShot; `Card`. Additionally there is
a `Range` class which represents basic information about a range.

## The `Card` object
The `Card` object essentially represents all you need know in order to draw a
single lane. It has information about the shooter, the target configuration and
the result.

Here is an example of a complete `Card` object:

```javascript
{
   "lane":1,
   "config":{
      "gaugeSize":0.0695,
      "targetID":"NO_DFS_15M"
   },
   "result":{
      "marking":false,
      "seriesName":"Prøve Kne",
      "seriesSum":"18",
      "shots":[ 
         {
            "value":"8.4",
            "x":-0.265175,
            "y":-0.001175
         },
         {
            "value":"X.1",
            "x":-0.07365,
            "y":-0.016
         }
      ],
      "totalSum":"0"
   },
   "shooter":{
      "category":"",
      "className":"5",
      "club":"Ullensaker",
      "name":"Kjell Aarskog"
   }
}
```

### Formal definition
**Card**  
* < _string_ >`lane`  
    The display name for the lane, ex: `'1'`
* < _Shooter_ >`shooter`  
    See **Shooter** below.
* < _Result_ >`result`  
    See **Result** below.
* < _CardConfig_ >`config`  
    See **CardConfig** below.

**Shooter**  
* < _string_ >`name`  
    The display name for the shooter, ex: `'Martin V. Larsen'`
* < _string_ >`club`  
    The display name of the club of the shooter, ex: `'Rygge'`
* < _string_ >`className`  
    The class of the shooter, ex: `'V55'`
* < _string_ >`category`  
    The the category of the shooter, ex: `'LF'`

**Result**  
* < _string_ >`seriesName`  
    The display name for the series, ex: `'Prone'`
* < _string_ >`seriesSum`
    The sum of the current series, formatted for display, ex: `'50'`
* < _string_ >`totalSum`
    The total sum of the card, formatted for display, ex: `'150'`
* < _boolean_ >`marking`
    Indicates whether this is a marking series (i.e. 'tellende'). Used for
    displaying test triangle on test series.
* < _iterable_ >`shots`
    * Contains zero or more < _Shot_ > objects, the keys of this object can be
    anything, as long as they iterate in the correct order. Typically these
    keys are just normal numeric indices as in an array.

**Shot**  
* < _number_ >`x`  
    The x-coordinate of the _center_ of the shot. This should be
    normalized, such that -1 <= `x` <= 1.
* < _number_ >`y`  
    The y-coordinate of the _center_ of the shot. This should be
    normalized, such that -1 <= `y` <= 1.
* < _string_ >`value`  
    `value` is the shot value, appropriately formatted for display,
    ex: `'X.0'`

**CardConfig**  
* < _number_ >`gaugeSize`  
    The size of the gauge (similar to diameter of the bullet) relative to
    the target size. For instance, if the target diameter is `1000mm`, and
    the gauge diameter is `8mm`, the value of `gaugeSize` should be `8mm /
    1000mm = .008`.
* < _string_ >`targetID`
    For valid values of `targetID`, see [List of implemented targets](https://github.com/martinvl/liveshot#list-of-implemented-targets).

## The `Range` object
`Range` objects represents metadata about a range and used in the `RangeView`.
Currently these don't really contain much data, but are intended to be extended
with timer data among other things in the future.

Here is a sample `Range` object:

```javascript
{
   "host":"Rygge SKL",
   "name":"300m",
   "relay":"1"
}
```

**Range**  
* < _string_ >`host`
    The display name of the host of the shooting. This is typically the name of
    the hosting club, ex: `'Rygge SKL'`
* < _string_ >`name`
    The range display name, ex: `'100m'`
* < _string_ >`relay`
    The display name for the relay, ex: `'1'`

## Builders
You can either build the above objects yourself, or use built-in
Builder-classes to do the work for you.

**CardBuilder**  
Builds `Card` objects. All setters return reference to the builder, for convenience.

```javascript
var card = new LiveShot.CardBuilder()
    .setLane('1')
    .setName('Martin V. Larsen')
    .setClub('Rygge')
    .setClassName('4')
    .setCategory('A')
    .setSeriesName('Ligg')
    .addShotData(.1, 0, 'X.0')
    .addShotData(-.1, 0, 'X.0')
    .addShotData(0, .1, 'X.0')
    .addShotData(0, -.1, 'X.0')
    .addShotData(0, 0, '*.9')
    .setSeriesSum('50')
    .setTotalSum('150')
    .setMarking(false)
    .setGaugeSize(.00533)
    .setTargetID('NO_DFS_300M')
    .getCard();
```
* (static method) **CardBuilder.createBlankCard**() - ( _Card_ )  
    Creates and returns a new empty card, with all fields present, but set to
    empty placeholder values.
* (static method) **CardBuilder.sanitizeCard**(< _object_ >rawCard) - ( _Card_ )  
    Creates a new `Card` object and copies all relevant fields from
    `rawCard`, omitting all non-relevant fields and inserting default values
    for missing fields. The returned object is guaranteed to be a valid `Card`
    object.
* **reset**() - ( _CardBuilder_ )  
    Resets the current card. Returns pointer to the builder for convenience.
* **getCard**() - ( _Card_ )  
    Returns a copy of the current card
* **setLane**(< _string_ >lane) - ( _CardBuilder_ )
* **setName**(< _string_ >name) - ( _CardBuilder_ )
* **setClub**(< _string_ >club) - ( _CardBuilder_ )
* **setClassName**(< _string_ >className) - ( _CardBuilder_ )
* **setCategory**(< _string_ >category) - ( _CardBuilder_ )
* **setSeriesName**(< _string_ >seriesName) - ( _CardBuilder_ )
* **setSeriesSum**(< _string_ >seriesSum) - ( _CardBuilder_ )
* **setTotalSum**(< _string_ >totalSum) - ( _CardBuilder_ )
* **setMarking**(< _boolean_ >marking) - ( _CardBuilder_ )
* **setGaugeSize**(< _number_ >gaugeSize) - ( _CardBuilder_ )
* **setTargetID**(< _string_ >targetID) - ( _CardBuilder_ )  
    For valid values of `targetID`, see [List of implemented
    targets](https://github.com/martinvl/liveshot#list-of-implemented-targets).
* **setShooter**(< _Shooter_ >shooter) - ( _CardBuilder_ )  
    `shooter` should be as described above
* **setResult**(< _Result_ >result) - ( _CardBuilder_ )  
    `result` should be as described above
* **setConfig**(< _CardConfig_ >config) - ( _CardBuilder_ )  
    `config` should be as described above
* **setShots**(< _iterable_ >shots) - ( _CardBuilder_ )  
    `shots` should be as described above
* **resetShots**() - ( _CardBuilder_ )  
    Resets the current list of shots
* **addShot**(< _Shot_ >shot) - ( _CardBuilder_ )  
    `shot` should be as described above
* **addShotData**(< _number_ >x, < _number_ >y, < _string_ >value) - ( _CardBuilder_ )  
    -1 <= `x`, `y` <= 1, where (`x`, `y`) represent the _center_ of the shot  
    `value` is the shot value, appropriately formatted for displaying

**RangeBuilder**  
Builds `Range` objects. All setters return reference to the builder, for convenience.

```javascript
var range = new LiveShot.RangeBuilder()
    .setHost('Rygge SKL')
    .setName('300m')
    .setRelay('1')
    .setCards(cards)
    .getRange();
```

* (static method) **RangeBuilder.createBlankRange**() - ( _Range_ )  
    Creates and returns a new empty range, with all fields present, but set to
    empty placeholder values.
* (static method) **RangeBuilder.sanitizeRange**(< _object_ >rawRange) - ( _Range_ )  
    Creates a new `Range` object and copies all relevant fields from
    `rawRange`, omitting all non-relevant fields and inserting default values
    for missing fields. The returned object is guaranteed to be a valid `Range`
    object.
* **reset**() - ( _RangeBuilder_ )  
    Resets the current range. Returns pointer to the builder for convenience.
* **getRange**() - ( _Range_ )  
    Returns a copy of the current range
* **setHost**(< _string_ >host) - ( _RangeBuilder_ )
* **setName**(< _string_ >range) - ( _RangeBuilder_ )
* **setRelay**(< _string_ >relay) - ( _RangeBuilder_ )

**ShooterBuilder**  
Builds `Shooter` objects. All setters return reference to the builder, for convenience.

```javascript
var shooter = new LiveShot.ShooterBuilder()
    .setName('Martin V. Larsen')
    .setClub('Rygge')
    .setClassName('4')
    .setCategory('A')
    .getShooter();
```

* (static method) **ShooterBuilder.createBlankShooter**() - ( _Shooter_ )  
    Creates and returns a new empty shooter object, with all fields present,
    but set to empty placeholder values.
* (static method) **ShooterBuilder.sanitizeShooter**(< _object_ >rawShooter) - ( _Shooter_ )  
    Creates a new `Shooter` object and copies all relevant fields from
    `rawShooter`, omitting all non-relevant fields and inserting default values
    for missing fields. The returned object is guaranteed to be a valid
    `Shooter` object.
* **reset**() - ( _ShooterBuilder_ )  
    Resets the current shooter object.
* **getShooter**() - ( _Shooter_ )  
    Returns a copy of the current shooter object
* **setShooter**(< _object_ >shooter) - ( _ShooterBuilder_ )  
    Copies all relevant fields from `shooter`
* **setName**(< _string_ >name) - ( _ShooterBuilder_ )
* **setClub**(< _string_ >club) - ( _ShooterBuilder_ )
* **setClassName**(< _string_ >className) - ( _ShooterBuilder_ )
* **setCategory**(< _string_ >category) - ( _ShooterBuilder_ )

**ResultBuilder**  
Builds `Result` objects. All setters return reference to the builder, for convenience.

```javascript
var result = new LiveShot.ResultBuilder()
    .setSeriesName('Ligg')
    .setSeriesSum('50')
    .setTotalSum('150')
    .setMarking(false)
    .addShot(shot)
    .addShotData(0, -.1, 'X.0')
    .addShotData(.071, .071, 'X.0')
    .addShotData(.071, -.071, 'X.0')
    .addShotData(-.071, .071, 'X.0')
    .getResult();
```

* (static method) **ResultBuilder.createBlankResult**() - ( _Result_ )  
    Creates and returns a new empty result object, with all fields present,
    but set to empty placeholder values.
* (static method) **ResultBuilder.sanitizeResult**(< _object_ >rawResult) - ( _Result_ )  
    Creates a new `Result` object and copies all relevant fields from
    `rawResult`, omitting all non-relevant fields and inserting default values
    for missing fields. The returned object is guaranteed to be a valid
    `Result` object.
* **reset**() - ( _ResultBuilder_ )  
    Resets the current result object.
* **getResult**() - ( _Result_ )  
    Returns a copy of the current result object
* **setResult**(< _Result_ >result) - ( _ResultBuilder_ )  
    Copies all relevant fields from `result`
* **setSeriesName**(< _string_ >seriesName) - ( _ResultBuilder_ )
* **setSeriesSum**(< _string_ >seriesSum) - ( _ResultBuilder_ )
* **setTotalSum**(< _string_ >totalSum) - ( _ResultBuilder_ )
* **setMarking**(< _boolean_ >marking) - ( _ResultBuilder_ )
* **setShots**(< _iterable_ >shots) - ( _ResultBuilder_ )  
    `shots` should be as described above
* **resetShots**() - ( _ResultBuilder_ )  
    Resets the current list of shots
* **addShot**(< _Shot_ >shot) - ( _ResultBuilder_ )  
    `shot` should be as described above
* **addShotData**(< _number_ >x, < _number_ >y, < _string_ >value) - ( _ResultBuilder_ )  
    -1 <= `x`, `y` <= 1, where (`x`, `y`) represent the _center_ of the shot  
    `value` is the shot value, appropriately formatted for displaying

**ShotBuilder**  
Builds `Shot` objects. All setters return reference to the builder, for convenience.

```javascript
var shot = new LiveShot.ShotBuilder()
    .setPosition(.1, 0)
    .setValue('X.0')
    .getShot();
```

* (static method) **ShotBuilder.createBlankShot**() - ( _Shot_ )  
    Creates and returns a new empty shot object, with all fields present,
    but set to empty placeholder values.
* (static method) **ShotBuilder.sanitizeShot**(< _object_ >rawShot) - ( _Shot_ )  
    Creates a new `Shot` object and copies all relevant fields from
    `rawShot`, omitting all non-relevant fields and inserting default values
    for missing fields. The returned object is guaranteed to be a valid
    `Shot` object.
* **reset**() - ( _ShotBuilder_ )  
    Resets the current shot object.
* **getShot**() - ( _Shot_ )  
    Returns a copy of the current shot object
* **setValue**(< _string_ >value) - ( _ShotBuilder_ )
* **setPosition**(< _number_ >x, < _number_ >y) - ( _ShotBuilder_ )  
    _x_ and _y_ should be normalized (i.e. `-1 <= x, y <= 1`)
* **setX**(< _number_ >x) - ( _ShotBuilder_ )  
    _x_ should be normalized (i.e. `-1 <= x <= 1`)
* **setY**(< _number_ >y) - ( _ShotBuilder_ )  
    _y_ should be normalized (i.e. `-1 <= y <= 1`)

**ConfigBuilder**  
Builds `Config` objects. All setters return reference to the builder, for convenience.

```javascript
var config = new LiveShot.ConfigBuilder()
    .setGaugeSize(.00533)
    .setTargetID('NO_DFS_300M')
    .getConfig();
```

* (static method) **ConfigBuilder.createBlankConfig**() - ( _CardConfig_ )  
    Creates and returns a new empty config object, with all fields present,
    but set to empty placeholder values.
* (static method) **ConfigBuilder.sanitizeConfig**(< _object_ >rawConfig) - ( _Config_ )  
    Creates a new `Config` object and copies all relevant fields from
    `rawConfig`, omitting all non-relevant fields and inserting default values
    for missing fields. The returned object is guaranteed to be a valid
    `Config` object.
* **reset**() - ( _ConfigBuilder_ )  
    Resets the current config object.
* **getConfig**() - ( _CardConfig_ )  
    Returns a copy of the current config object
* **setConfig**(< _object_ >config) - ( _ConfigBuilder_ )  
    Copies all relevant fields from `config`
* **setGaugeSize**(< _number_ >gaugeSize) - ( _ConfigBuilder_ )
* **setTargetID**(< _string_ >targetID) - ( _ConfigBuilder_ )  
    For valid values of `targetID`, see [List of implemented targets](https://github.com/martinvl/liveshot#list-of-implemented-targets).
