var inherits = require('inherits');

var Renderer = require('./Renderer');
var CardRenderer = require('./CardRenderer');
var RangeBuilder = require('../protocol/RangeBuilder');

function RangeRenderer() {
    Renderer.prototype.constructor.apply(this);

    this.initialize();
}

module.exports = RangeRenderer;
inherits(RangeRenderer, Renderer);

// --- External API ---
RangeRenderer.prototype.setStyle = function (style) {
    for (var key in style) {
        this.rawStyle[key] = style[key];
    }

    for (var key in this.style) {
        if (style.hasOwnProperty(key))
            this.style[key] = style[key];
    }

    for (var idx in this.cardRenderers) {
        this.cardRenderers[idx].setStyle(style);
    }

    return this;
};

RangeRenderer.prototype.setRange = function (range, valid) {
    if (!valid) {
        range = RangeBuilder.sanitizeRange(range);
    }

    this.range = range;

    return this;
};

RangeRenderer.prototype.setCards = function (cards, valid) {
    // add new cards
    for (var idx in cards) {
        this.setCard(idx, cards[idx], valid, true);
    }

    // remove old new cards
    for (var idx in this.cardRenderers) {
        if (!cards.hasOwnProperty(idx)) {
            this.removeCard(idx, true);
        }
    }

    this.pushCardRects();

    return this;
};

RangeRenderer.prototype.setCard = function (idx, card, valid, maintainRects) {
    if (!this.cardRenderers.hasOwnProperty(idx)) {
        this.addCardRenderer(idx);

        if (!maintainRects) {
            this.pushCardRects();
        }
    }

    this.cardRenderers[idx].setCard(card, valid);

    return this;
};

RangeRenderer.prototype.removeCard = function (idx, maintainRects) {
    this.removeCardRenderer(idx);

    if (!maintainRects) {
        this.pushCardRects();
    }

    return this;
};

// --- Internal API ---
RangeRenderer.prototype.initialize = function () {
    this.cardRenderers = {};
    this.rawStyle = {};
    this.style = {};
    this.numCardRenderers = 0;

    this.setRange(RangeBuilder.createBlankRange());
};

RangeRenderer.prototype.setContext = function (context) {
    var res = Renderer.prototype.setContext.apply(this, [context]);

    for (var idx in this.cardRenderers) {
        this.cardRenderers[idx].setContext(this.context);
    }

    return res;
};

RangeRenderer.prototype.setRect = function (rect) {
    var res = Renderer.prototype.setRect.apply(this, [rect]);

    this.pushCardRects();

    return res;
};

RangeRenderer.prototype.addCardRenderer = function (idx) {
    var renderer = new this.CardRenderer();
    renderer.setContext(this.context);
    renderer.setStyle(this.rawStyle);

    this.cardRenderers[idx] = renderer;

    ++this.numCardRenderers;
};

RangeRenderer.prototype.removeCardRenderer = function (idx) {
    delete this.cardRenderers[idx];
    --this.numCardRenderers;
};

RangeRenderer.prototype.draw = function () {
    for (var idx in this.cardRenderers) {
        this.cardRenderers[idx].render();
    }
};

RangeRenderer.prototype.pushCardRects = function () {
    var rect = this.getCardsRect();
    var layout = this.getCardsLayout();

    var cardNum = 0;
    var cardIndices = this.getCardIndices();
    var cardWidth = rect.width/layout.numCols;
    var cardHeight = rect.height/layout.numRows;

    for (var i = 0; i < layout.numRows; ++i) {
        for (var j = 0; j < layout.numCols; ++j) {
            if (cardNum >= cardIndices.length) {
                return;
            }

            var cardIdx = cardIndices[cardNum++];

            var leftEdge = Math.floor(j*cardWidth) + rect.x;
            var rightEdge = Math.floor((j + 1)*cardWidth) + rect.x;
            var topEdge = Math.floor(i*cardHeight) + rect.y;
            var bottomEdge = Math.floor((i + 1)*cardHeight) + rect.y;

            this.cardRenderers[cardIdx].setRect({
                x: leftEdge,
                y: topEdge,
                width: rightEdge - leftEdge,
                height: bottomEdge - topEdge
            });
        }
    }
};

RangeRenderer.prototype.getCardIndices = function () {
    var indices = [];

    for (var idx in this.cardRenderers) {
        indices.push(idx);
    }

    return indices;
};

RangeRenderer.prototype.CardRenderer = CardRenderer;

// --- Layout utils ---
RangeRenderer.prototype.getCardsRect = function () {
    return this.rect;
};

RangeRenderer.prototype.getCardsLayout = function () {
    var rect = this.getCardsRect();
    var N = this.numCardRenderers;
    var minBadness = Number.MAX_VALUE;
    var bestM = 0;
    var bestN = 0;

    if (N == 5 || N == 7) {
        ++N;
    }

    for (var m = 1; m <= N; ++m) {
        var n = Math.ceil(N / m);
        var cardWidth = rect.width/n;
        var cardHeight = rect.height/m;
        var targetRect = this.CardRenderer.getTargetRect({x: 0, y: 0, width: cardWidth, height: cardHeight});

        var aspectRatio = targetRect.width / targetRect.height;
        var optimalRatio = 1;
        var ratioBadness =  Math.abs(aspectRatio - optimalRatio) + Math.abs(1/aspectRatio - 1/optimalRatio);
        var cellCountBadness = Math.abs(n*m - N);
        var badness = ratioBadness + cellCountBadness;

        if (badness < minBadness) {
            minBadness = badness;
            bestM = m;
            bestN = n;
        }
    }

    return {
        numCols: bestN,
        numRows: bestM
    };
};
