var TargetRenderer = require('./TargetRenderer');

function RingTargetRenderer() {
    TargetRenderer.prototype.constructor.apply(this);

    this.style.drawFullTarget = false;
    this.style.ringLineWidth = .75;
}

RingTargetRenderer.prototype = new TargetRenderer();
RingTargetRenderer.prototype.constructor = RingTargetRenderer;
module.exports = RingTargetRenderer;

// --- External API ---
RingTargetRenderer.prototype.setTarget = function (target) {
    this.target = target;

    return this;
};

// --- Internal API ---
RingTargetRenderer.prototype.drawTarget = function () {
    this.drawBackground();
    this.drawRings();
    this.drawNumbers();
};

RingTargetRenderer.prototype.drawBackground = function () {
    var ctx = this.context;
    var size = this.getSize();

    // draw back
    var backSize = this.getMaxSize();
    ctx.fillStyle = this.style.backColor;

    ctx.beginPath();
    ctx.arc(0, 0, backSize * size, 0, Math.PI*2, true);
    ctx.closePath();
    ctx.fill();

    // draw front
    var frontSize = Math.min(this.target.frontSize, this.getMaxSize());
    ctx.fillStyle = this.style.frontColor;

    ctx.beginPath();
    ctx.arc(0, 0, frontSize * size, 0, Math.PI*2, true);
    ctx.closePath();
    ctx.fill();

    // draw sight lines
    if (this.target.drawSightLines) {
        ctx.beginPath();
        ctx.fillStyle = this.style.backColor;
        ctx.fillRect(-frontSize*size, -0.01*frontSize*size, frontSize*size/2, 0.02*frontSize*size);
        ctx.fillStyle = this.style.backColor;
        ctx.fillRect(frontSize*size/2, -0.01*frontSize*size, frontSize*size/2, 0.02*frontSize*size);
        ctx.closePath();
        ctx.fill();
    }
};

RingTargetRenderer.prototype.drawRings = function () {
    var ctx = this.context;
    var frontSize = this.target.frontSize;
    var ringSizes = this.target.ringSizes;
    var size = this.getSize();
    var pixelRatio = window.devicePixelRatio || 1;

    for (var idx in ringSizes) {
        var ringSize = ringSizes[idx];

        if (ringSize == frontSize || ringSize > this.getMaxSize()) {
            continue;
        }

        var radius = ringSize*size;
        var isFrontRing = ringSize <= frontSize;
        ctx.strokeStyle = isFrontRing
            ? this.style.backColor
            : this.style.frontColor;
        ctx.lineWidth = this.style.ringLineWidth*pixelRatio;

        ctx.beginPath();
        ctx.arc(0, 0, radius, 0, Math.PI*2, true);
        ctx.closePath();

        ctx.stroke();

        if (isFrontRing && this.target.drawSightLines && ringSize > 0.5*frontSize) {
            var dh = 0.01*frontSize*size;
            var alpha = Math.asin(dh/radius);

            ctx.strokeStyle = this.style.frontColor;
            ctx.beginPath();
            ctx.arc(0, 0, radius, -alpha, alpha, false);
            ctx.closePath();

            ctx.stroke();

            ctx.beginPath();
            ctx.arc(0, 0, radius, Math.PI - alpha, Math.PI + alpha, false);
            ctx.closePath();

            ctx.stroke();
        }
    }
};

RingTargetRenderer.prototype.drawNumbers = function () {
    var ctx = this.context;
    var size = this.getSize();

    for (var i = 0; i + 1 < this.target.ringSizes.length
            && i + this.target.numbersFrom <= this.target.numbersTo; ++i) {
        var number = i + this.target.numbersFrom;
        var lowerRingSize = this.target.ringSizes[i];
        var upperRingSize = this.target.ringSizes[i + 1];

        if (lowerRingSize > this.getMaxSize()) {
            continue;
        }

        var ringMid = (lowerRingSize + upperRingSize)/2;
        var d = ringMid*size;

        if (ringMid < this.target.frontSize) {
            ctx.fillStyle = this.style.backColor;
        } else {
            ctx.fillStyle = this.style.frontColor;
        }

        if (this.target.drawHorisontalNumbers) {
            this.drawNumber(number, -d, 0);
            this.drawNumber(number, d, 0);
        }

        if (this.target.drawVerticalNumbers) {
            this.drawNumber(number, 0, -d);
            this.drawNumber(number, 0, d);
        }

        if (this.target.drawDiagonalNumbers) {
            var cat = d/Math.sqrt(2);

            this.drawNumber(number, -cat, -cat);
            this.drawNumber(number, -cat, cat);
            this.drawNumber(number, cat, -cat);
            this.drawNumber(number, cat, cat);
        }
    }
};

RingTargetRenderer.prototype.drawNumber = function (number, dx, dy) {
    var ctx = this.context;
    var size = this.getSize();

    ctx.font = "36px arial";
    ctx.textBaseline = "middle";
    ctx.textAlign = "center";

    ctx.save();
    ctx.translate(dx, dy);
    ctx.scale(1/500 * size, 1/500 * size);

    ctx.fillText(number, 0, 0);

    ctx.restore();
};

RingTargetRenderer.prototype.getSize = function () {
    return this.scale * Math.min(this.rect.width, this.rect.height)/2;
};

RingTargetRenderer.prototype.getMaxSize = function () {
    var maxSize = 1;

    if (!this.style.drawFullTarget)
        maxSize = Math.min(maxSize, 1 / this.scale);

    return maxSize;
};
